# **************************************************
#
#  The kraken entry data
#
# **************************************************

import time
import sys
import os.path
import string
import strptime
import traceback
import logging
from rdflib.URIRef import URIRef
import kraken.entrytypes.MetaDataDescriptor as MetaDataDescriptor
import kraken.tools.TextUi as TextUi
import kraken.Constants
import kraken.entrytypes.Utilities
import kraken.platform.PlatformUtilities as PlatformUtilities
from EntryBase import EntryBase
from kraken.tools.UniqueIdGenerator import getUniqueId
from TypicalNamespaces import *


class Entry(EntryBase):            
    '''
    The entry class
    '''
    
# RDF mappings of the id's in the class
    mappings = {}
    mappings["label"]          = NS_KRAKEN["label"]
    mappings["directoryName"]  = NS_KRAKEN["directoryName"]
    mappings["title"]          = NS_DC["title"]
    mappings["description"]    = NS_DC["description"]
    mappings["date"]           = NS_DC["date"]
    mappings["localFileNames"] = NS_KRAKEN["localFileNames"]
    mappings["originalURIs"]   = NS_KRAKEN["originalURIs"]
    mappings["keywords"]       = NS_KRAKEN["keywords"]
    mappings["entryTypes"]     = NS_KRAKEN["entryTypes"]
    
    def __init__(self):
       EntryBase.__init__(self)
       # the label. This is a fictive URL: kraken://
       # self.label = "kraken://" + getUniqueId()
       self.registerProperty("label", "#" + getUniqueId())
       # the name of the directory inside the kraken. This is considered
       # to be mildly significant
       self.registerProperty("directoryName", "")
       # the title of the enty
       self.registerProperty("title", "")
       # short textual description of the entry
       self.registerProperty("description", "")
       # the date when it was created
       self.registerProperty("date", time.localtime())
       # the name of top level files. If there is more than one, it should
       # as a continuation. There can be extra files (icons etc) which do not
       # appear here
       self.registerProperty("localFileNames", [])
       # the original URLs of the entries
       self.registerProperty("originalURIs", [])
       # the keywords
       self.registerProperty("keywords", []) # comma separated
       # the entry types
       self.registerProperty("entryTypes", []) # comma separated
       
       # Non-persistent data (is not serialized in RDF)       
       # the kraken dir name (for multiple directories)
       self.parentDirName = ""
       self.secondaries = {}
           
    def getResourceURI(self):
        return URIRef(self.getProperty("label"))

    def getMapping(self, field):
        return self.mappings[field]

    def getRdfClass(self):
        return URIRef(NS_KRAKONT["KrakenEntry"])                    
        
     # **************************************************
     #   fillin data from runtime
     #   this refers to the annotation data, not the original stuff, etc
     # **************************************************
    def fillinEntryFromRuntime(self, runtime, system, force = 0): 
        self.addProperty("directoryName", runtime.entryDir)
        if self.parentDirName == "":
            self.parentDirName = system.getProperty("destinationDir") 
        if force or runtime.title != "":
            self.addProperty("title", runtime.title)
        if force or runtime.description != "":     
            self.addProperty("description", runtime.description)
        if force or runtime.keywords != []:  
            self.addProperty("keywords", runtime.keywords)
        if force or runtime.date != None:
            self.addProperty("date", runtime.date)

    def edit(self, metadata, isAnnotation = 1, withConfirm = 0):
        """
            Edit the entry  
        """    
        if not isAnnotation: # meaning creation time
           self.editPrimary(metadata, isAnnotation, withConfirm)
        while 1:
            items = ["Primary data"]
            descriptors = self.getApplicableMetaDataDescriptors(metadata)
            # add the descriptors which are anyhow there
            for a in metadata.descriptors.values():
                if a.existsInEntry(self) and (a not in descriptors):
                    descriptors.append(a)
            for a in descriptors:
                items.append(a.getProperty("name"))
            items.append("-")
            items.append("Open directory of the entry")
            items.append("Print the entry")
            items.append("Accept as it is")
            val = TextUi.inputMenu("Which part of the entry do you want to edit?", items, default="Primary data")
            if val == "Accept as it is":
                return
            if val == "Print the entry":
                print self
                # FIXME: indentation
                for a in self.secondaries.keys():
                    print "\n"
                    print self.secondaries[a]
                print "\n\n"
                continue
            if val == "Open directory of the entry":
                PlatformUtilities.openDirectory(
                   os.path.join(self.parentDirName, self.getProperty("directoryName")))
            if val == "Primary data":
                self.editPrimary(metadata, isAnnotation, withConfirm)
                continue
            for a in descriptors:                
                if val == a.getProperty("name"):
                   try: 
                      a.editEntry(self, confirmCreation = 0)
                      break
                   except:
                      traceback.print_exc()
                      print "There was an exception while editing secondary "+ val
                      break
    
    
    def getExtendedProperty(self, propertyname):
        props = propertyname.split(".")
        if len(props) == 1:
            return self.getProperty(propertyname)
        elif len(props) == 2:
            secondaryName = props[0]
            #print "Looking up in " + secondaryName + " value " + props[1]
            try:
               theSecondary = self.secondaries[secondaryName]
            except KeyError:                
               return None
            #print self.getProperty("title") + "Found up in " + secondaryName + " value " + props[1] + str(theSecondary.getProperty(props[1]))            
            return theSecondary.getProperty(props[1])
            
    def addExtendedProperty(self, propertyname, value):
        props = propertyname.split(".")
        if len(props) == 1:
            self.addProperty(self, propertyname, add)
        elif len(props) == 2:
            secondaryName = props[0]
            theSecondary = secondaries[secondaryName]
            theSecondary.addProperty(props[1])
    
    def listExtendedPropertyNames(self):
        lst = self.listPropertyNames()
        for key in self.secondaries.keys():
            sec = self.secondaries[key]
            for p in sec.listPropertyNames():
                lst.append(key + "." + p)
        return lst
    
    
    #
    #  Command line based editing of an entry - mainly for annotation, etc
    #            
    def editPrimary(self, metadata, isAnnotation = 1, withConfirm = 0):
        print "Label = " + self.getProperty("label")
        self.editField("title", "Title")
        self.editField("description", "Description")
        kw = self.getProperty("keywords")
        if metadata != None:
           kwnew = TextUi.inputSequence("Keywords", kw, metadata.keywords)
        else:
           kwnew = TextUi.inputSequence("Keywords", kw)
        self.addProperty("keywords", kwnew)
        # the select files is only for the annotation step, because at first is not present
        lf = self.getProperty("localFileNames")        
        if isAnnotation:   
          lfn = TextUi.selectFilesFromDirectory(
                   self.getAbsoluteDirName(), lf, ignore=["Metadata"])
        self.addProperty("localFileNames", lf)
        # the originalURI's
        self.editField("originalURIs", "Remote URI's")
        self.editField("date", "Date")
        self.addProperty("entryTypes",
           TextUi.inputEntryTypes(metadata, default = self.getProperty("entryTypes"))
           )
      
   
    """
       Editing the secondary stuff
    """
    def getApplicableMetaDataDescriptors(self, metadata):
        descriptors = []
        for key in metadata.descriptors.keys():
           #print key
           descriptor = metadata.descriptors[key]
           entryTypes = self.getProperty("entryTypes") 
           #print entryTypes
           applicable = descriptor.getProperty("applicableToEntryTypes")
           app = False
           #print key, " Applicable ", applicable
           if "*" in applicable:
               app = True
           else:  
               for a in applicable:
                  if a in entryTypes:
                     app = True
                     break
           if app:
              descriptors.append(descriptor)
        return descriptors
           
    #
    #  Checks if the metadata dir exists and creates it if it doesn't
    #
    def __prepareMetadataDir(self, dirName):
        metadataDir = os.path.join(dirName, kraken.Constants.DIRNAME_METADATA)
        if os.path.isdir(metadataDir):
            return
        os.mkdir(metadataDir)       
       
    def writeRDF(self, dirName):
       self.__prepareMetadataDir(dirName)
       self.writeEntryToRdf(getEntryRdfFile(dirName))
       
    def getAbsoluteDirName(self):
        return os.path.join(self.parentDirName, self.getProperty("directoryName"))
       
              
def _getEntryAuxFileName(dirName, filename):
    return os.path.join(dirName, kraken.Constants.DIRNAME_METADATA, filename)
        
def getEntryRdfFile(dirName):
    return _getEntryAuxFileName(dirName, kraken.Constants.FILENAME_ENTRY_RDF)
        
#
#  Loading an entry from the file
#        
def loadKraEntry(system, dirName, subDirName):
  try:
    fileToOpen = getEntryRdfFile(os.path.join(dirName, subDirName))
    temp = Entry()
    temp.parentDirName = dirName
    temp.readEntryFromRdf(fileToOpen)
    descriptors = MetaDataDescriptor.getMetaDataDescriptors()
    for d in descriptors.keys():
           #print "Attempt to read data: " + d
           value = descriptors[d].loadSecondary(temp)
           if value != None:
              temp.secondaries[d] = value
    return temp
  except:
    traceback.print_exc()
    return None

    
